$!Pestfind.com
$! 4-NOV-1995, David Mathog, biology division, Caltech
$! 28-JUN-1996 Modified so that it always looks for a ".."
$!    in list files, to match changes in GCG to that effect.
$! 18-SEP-1996 Modified so that it can read a .seqcat file and
$!    use it as a list file - one quick way to search a whole database.
$!    Also, an option to only look at POTENTIAL pestfind scores
$!
$! This procedure is a wrapper for the actual PestFind program, it
$! allows it to act on one or more GCG formatted files.
$! This uses the TOSEQ program, if you don't have that, you may
$! have the EGCG TOTEXT, which does the same thing.
$!
$! All files are processed with a window of 9 and no invalid PEST sequences.
$!
$! A logical PESTFIND must point to the location of the PESTFIND program
$!
$! invoke with 
$!
$! @pestfind filename        (either a file or a GCG entry)
$!       or
$! @pestfind @filename       (a file of file names)
$!       or
$! @pestfind @wherever:yp.seqcat yp  (use all entries in a seqcat) 
$!
$ if(P1 .eqs. "")
$ then
$   type sys$input

    Find PEST sequences in proteins.


    This will accept as input one of:

    1.  The name of a GCG formatted peptide file:         filename.pep
    2.  The name of a GCG peptide database entry:         sw:1433_HORVU
    3.  A GCG file of filenames (containing either
        of the above but nothing else)                    @filelist.fil
    4.  The path to a GCG seqcat file and a prefix for the
        database:
                 @pirdir:pir1.seqcat pir1

Note 1:  If any sequence contains an ambiguous residue, such as B,X, or Z,
         pestfind will not process that sequence.  It will issue an error
         message and stop.

Note 2:  Default Parameters:  minimum pest sequence (window) size =9
                              no invalid PEST sequences shown
                              outfile = pestfind.out

         To change one or more of thse parameters, hit a ^Y NOW and issue
         one or more of these commands before running PESTFIND again:

         $ pestfind_window  :== 10         (or whatever value you want)
         $ pestfind_invalid :==  Y         (to see invalid ones) 
         $ pestfind_out     :==  filename  (valid VMS filename) 
       
$   inquire acton "Enter file, entry, or file of filenames = "
$ else
$    acton = P1
$ endif
$!
$ if (P2 .nes. "")
$ then
$   prefix = P2 + ":"
$ else
$   prefix = ""
$ endif
$!
$ if (P3 .nes. "")
$ then
$   potential = 1
$ else
$   potential = 0
$ endif
$!
$! make up a root for temporary file names
$!
$ now = f$time()
$ username = f$getjpi("","USERNAME")
$ username = f$edit(username,"COLLAPSE")
$ tempname = "kill_" + username + "_" -
              + f$cvtime(now,,"MONTH") -
              + f$cvtime(now,,"DAY") -
              + f$cvtime(now,,"HOUR") -
              + f$cvtime(now,,"MINUTE") -
              + f$cvtime(now,,"HUNDREDTH")
$!
$! does it begin with an "@"?
$!
$ on control_c then goto
$ if(f$extract(0,1,acton) .eqs. "@")
$ then
$   loop = 1
$   namelen = f$length(acton) - 1
$   open/read/error=oops tfil: 'f$extract(1,namelen,acton)
$!
$!   find the ".." and then read files after that
$!
$   if(prefix .eqs. "")
$   then
$     dots:
$        read/error=nodots/end=nodots tfil: file
$        if(file .eqs. file - "..")then goto dots
$   endif
$      read tfil: file
$      file = f$edit(file,"TRIM")
$      file = file + " "
$      file = f$edit(file,"COMPRESS")
$      file = f$element(0," ",file)
$ else
$   loop = 0
$   file = acton
$ endif
$!
$!
$ if("''pestfind_window'" .nes. "")
$ then
$   window = pestfind_window
$ else
$   window = 9
$ endif
$!
$ if("''pestfind_invalid'" .nes. "Y")
$ then
$   invalid = "N"
$ else
$   invalid = "Y"
$ endif
$!
$ if("''pestfind_out'" .nes. "")
$ then
$   pout  = pestfind_out
$ else
$   pout := pestfind.out
$ endif
$!
$  file = prefix + file
$open/write rfil: 'tempname'.accum
$write rfil: "Pestfind analysis of ''acton'"
$write rfil: "Processing began at ''now'"
$write rfil: " "
$write rfil: "Results on:  ''file'"
$write rfil: "===================================================="
$close rfil:
$!
$  on error then goto oops
$top:
$  write sys$output "Now processing:  ''file'"
$  toseq/infile='file'/out='tempname'.pep/line=50/protein/default
$  open/write cfil: 'tempname'.com
$  write cfil: "define/user sys$output nla0:"
$  write cfil: "define/user sys$error  ''tempname'.error"
$  write cfil: "$run pestfind:pestfind"
$  write cfil: "''tempname'.pep"
$  write cfil: window
$  write cfil: invalid
$  write cfil: "''tempname'.tmp"
$  close cfil:
$  @'tempname'.com
$  define/user sys$output nla0:
$  define/user sys$error  nla0:
$  search 'tempname'.error "PESTFIND run completed normally"
$  if($STATUS .ne. 1)
$  then
$!
$!    error during run, put the error message in instead
$!
$     convert/fdl=sys$input 'tempname'.error 'tempname'.tmp2 
RECORD
        BLOCK_SPAN              yes
        CARRIAGE_CONTROL        print
        FORMAT                  vfc
        SIZE                    0
        CONTROL_FIELD_SIZE      2
$  else
$!
$! Run completed ok, so tack the results on
$!
$    if(Potential .eq. 1)
$    then
$       search/out='tempname'.tmp3/window=(0,4) 'tempname'.tmp potential
$       delete 'tempname'.tmp.
$       rename 'tempname'.tmp3 .tmp
$    endif
$! Convert file type so that append will work properly.  streamlf
$! strikes again :-(.
$!
$     convert/fdl=sys$input 'tempname'.tmp 'tempname'.tmp2 
RECORD
        BLOCK_SPAN              yes
        CARRIAGE_CONTROL        print
        FORMAT                  vfc
        SIZE                    0
        CONTROL_FIELD_SIZE      2
$  endif
$  append 'tempname'.tmp2 'tempname'.accum
$  delete 'tempname'.tmp2;
$  delete 'tempname'.error;
$  delete 'tempname'.pep;
$  delete 'tempname'.com;
$  delete 'tempname'.tmp;
$  if(loop .eq. 1)
$  then
$     read/end=done tfil: file
$     file = f$edit(file,"TRIM")
$     file = file + " "
$     file = f$edit(file,"COMPRESS")
$     file = f$element(0," ",file)
$     file = prefix + file
$     open/append rfil: 'tempname'.accum
$     write rfil: "Results on:  ''file'"
$     write rfil: "===================================================="
$     close rfil:
$     goto top
$  endif
$  goto cleanup
$!
$done:
$  close tfil:
$!
$cleanup:
$  rename 'tempname'.accum 'pout'
$  write sys$output "Run completed, results are in ''pout'"
$  exit
$!
$oops:
$  write sys$output "--------------------------"
$  write sys$output "FATAL ERROR!!!!!"
$  write sys$output "something went wrong when processing ''acton'"
$  if(loop .eq. 1)then close tfil:
$  exit
$oops2:
$  write sys$output "--------------------------"
$  write sys$output "Run aborted by ^C or ^Y"
$  delete 'tempname'.*.*
$  exit
$nodots:
$  write sys$output "--------------------------"
$  write sys$output "FATAL ERROR!!!!"
$  write sys$output "There was no "".."" separator in the list file"
$  close tfil:
$  exit
